# Copyright(c) 1999, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: Maps.tcl,v 1.21 2004/02/28 17:33:26 tsmith Exp $

#################################################################################
# This Module handles configuring and drawing maps over images or over 	    	#
# empty frames																	#
#################################################################################
#
# InitMaps 	-- Sets Global variables and loads Context file data
# ConfigMap 	-- Used to set the variables so that when the map is drawn the
#		   same info can be reused
# DrawMap	-- Creates the Map command based on all of the graphics settings
#
# ToggleLALO	-- Displays/hides lat/lon settings
#
# DrawMap	-- Builds call to the mcidas MAP command
#
# GetMaps	-- Returns a list of all map definitions from the MAP.* context
#		   files.
# 
# FillMapMenu 	-- Populate the 'Map Name' menu. 
#
# FrameRange 	-- Enables/disables entry windows in frame selection section
#
# SearchContextFile -- Returns the value of the given variable in the given
#			   context file.  Returns the value of the first occurance
#			   of the given variable or 'NONE' if no variable was found.
#
# ReadContextFileVariable -- Returns the value of the given variable from the
#					 map context files.  Calls SearchContextFile for
#					 each file (MAP.USER, MAP.SITE, and MAP.CORE) in
#					 that order and takes the value of the first
#					 occurance of the variable.  Returns 'NONE' if no
#					 variable was found.
#
# IsImageInFrame -- Checks to see if an image is currently loaded into a given
#		    frame, returns YES or NO.
#
# ReadColorTable -- Reads the current color table values
#
# Restore Default Settings -- resets all map settings to their defaults
#
# LatLonOptions -- Creates lat/lon options section when check button is 
#		   selected for the first time.  Thereafter the options
#		   section is toggled on/off.
#
#################################################################################


#-------------------------------------------------------------------------------#
# InitMaps { } -- Sets global variables and loads from Context files as well    #
#-------------------------------------------------------------------------------#
proc InitMaps { } \
{
  global MapName 
  global MapCommandName
  global MapColor 
  global MapLALO 
  global MapLALOColor 
  global MapLALOLonInt 
  global MapLALOLatInt
  global MapInit
  global MapProjection
  global which_frames
  global map_beg_range
  global map_end_range
  
  #--- Return if variables have already been initialized in 
  #--- current McIDAS session
  if {[info exists MapInit]} {return}

  #--- For now intiialize to default values -- later will load from file  

  set MapName "Default Map"
  set MapCommandName "X"
  set MapLALO 0
  set MapLALOColor 2
  set MapLALOLonInt 30
  set MapLALOLatInt 30
  set MapInit 1
  set which_frames CURRENT

  # Get the default map color from the MAP.* context files
  set MapColor [ReadContextFileVariable MCOLOR]
  if {[string match $MapColor "NONE"]} {set MapColor 1}

  # Get default projection from context file
  set MapProjection [ReadContextFileVariable PROJECTION]
  if {[string match $MapProjection "NONE"]} {set MapProjection "MERC"}
}
  
#-------------------------------------------------------------------------------#
# ConfigMap { } -- procedure to select all of the map options					#
#-------------------------------------------------------------------------------#
proc ConfigMap { option } \
{
  global MapName 
  global MapCommandName
  global MapColor 
  global MapLALO 
  global MapLALOColor
  global MapLALOLonInt 
  global MapLALOLatInt
  global MapInit
  global MapProjection
  global which_frames
  global map_beg_range
  global map_end_range
  global num_colors
  global color_table
  

  # Initialize global variables (only upon startup of McIDAS)
  if { ![info exists MapInit] } {InitMaps}
  
  # If GUI already exists, bring to front
  if { [ winfo exists .tlMap] } \
  { 
	wm deiconify .tlMap
	focus .tlMap
	raise .tlMap
	return
  }
  
  toplevel .tlMap -borderwidth 3 -relief raised
  wm title .tlMap "Map Configuration"
  wm geometry .tlMap +100+100
  wm resizable .tlMap no no
  
  frame .tlMap.fmMain \
	-relief raised \
	-borderwidth 2
  pack .tlMap.fmMain \
	-side top \
	-padx 5 \
	-pady 5 \
	-fill x \
	-expand yes

  frame .tlMap.fmMain.fmNameMenu
  pack .tlMap.fmMain.fmNameMenu \
	-side top \
	-pady 5 \
	-padx 10 \
	-fill x \
	-expand yes

  set f .tlMap.fmMain.fmNameMenu

  menubutton $f.mb \
	-text "Map Name" \
	-menu $f.mb.menu \
	-relief raised \
	-borderwidth 2 \
	-indicatoron 1 \
	-width 12
  pack $f.mb -side left

  label $f.enNameEcho \
	-textvar MapName \
	-relief sunken \
	-borderwidth 2 \
	-anchor w
  pack $f.enNameEcho \
	-side left \
	-fill x \
	-expand true

  # Fill the menu for the Map Name menu button
  set m [FillMapMenu .tlMap.fmMain.fmNameMenu.mb.menu]

  # Add frame options ----------------------------------

  frame .tlMap.fmMain.fmWhichFrames \
	-relief groove \
	-borderwidth 2
  pack .tlMap.fmMain.fmWhichFrames \
	-side top \
	-fill x \
	-expand yes \
	-padx 10 \
	-pady 5

  radiobutton .tlMap.fmMain.fmWhichFrames.current \
	-text "Current Frame Only  " \
	-variable which_frames \
	-value CURRENT \
	-highlightthickness 0 \
	-command { FrameRange DISABLE }
   pack .tlMap.fmMain.fmWhichFrames.current \
	-side top \
	-anchor w \
	-padx 5 \
	-pady 5

   radiobutton .tlMap.fmMain.fmWhichFrames.setForLoop \
	-text "Set For Current Loop" \
	-variable which_frames \
	-value LOOP \
	-highlightthickness 0 \
	-command { FrameRange DISABLE }
   pack .tlMap.fmMain.fmWhichFrames.setForLoop \
	-side top \
	-anchor w \
	-padx 5 \
	-pady 5

   frame .tlMap.fmMain.fmWhichFrames.rangeFrame
   pack .tlMap.fmMain.fmWhichFrames.rangeFrame \
	-side top \
	-fill x \
	-expand yes \
	-padx 5 \
	-pady 5

   radiobutton .tlMap.fmMain.fmWhichFrames.rangeFrame.range \
 	-text "Range of Frames From: " \
	-variable which_frames \
	-value RANGE \
	-highlightthickness 0 \
	-command { FrameRange ENABLE }
   pack .tlMap.fmMain.fmWhichFrames.rangeFrame.range \
	-side left

   entry .tlMap.fmMain.fmWhichFrames.rangeFrame.enFrom \
	-width 3 \
	-state disabled \
	-background gray \
	-textvariable map_beg_range
   pack .tlMap.fmMain.fmWhichFrames.rangeFrame.enFrom \
	-side left

   label .tlMap.fmMain.fmWhichFrames.rangeFrame.lbTo \
	-text "to" \
	-highlightthickness 0
   pack .tlMap.fmMain.fmWhichFrames.rangeFrame.lbTo \
	-side left

   entry .tlMap.fmMain.fmWhichFrames.rangeFrame.enTo \
	-width 3 \
	-state disabled \
	-background gray \
	-textvariable map_end_range
   pack .tlMap.fmMain.fmWhichFrames.rangeFrame.enTo \
	-side left

   # Set state of radio buttons based on previous state
   if {[string match $which_frames "CURRENT"]} \
   {
	FrameRange DISABLE
   } \
   elseif {[string match $which_frames "RANGE"]} \
   {
	FrameRange ENABLE
   }

# Map color menu button -----------------------------------

   # Retrieve the current color table
   ReadColorTable

   frame .tlMap.fmMain.fmMapColor
   pack .tlMap.fmMain.fmMapColor \
	-side top \
	-pady 5 \
	-padx 10 \
	-anchor w

   label .tlMap.fmMain.fmMapColor.lbMapColor \
	-text "Map Color:"
   pack .tlMap.fmMain.fmMapColor.lbMapColor \
	-side left 
    
   # Map color menu button
   menubutton .tlMap.fmMain.fmMapColor.mapColorMenuButton \
	-text [ColorLevelToName $MapColor] \
	-background [ColorLevelToRGB $MapColor] \
	-activebackground [ColorLevelToRGB $MapColor] \
	-width 12 \
	-relief raised \
	-borderwidth 2 \
	-indicatoron yes \
	-menu .tlMap.fmMain.fmMapColor.mapColorMenuButton.menu
   pack .tlMap.fmMain.fmMapColor.mapColorMenuButton \
	-side left \
	-padx 5

   # Create and fill map color menu
   menu .tlMap.fmMain.fmMapColor.mapColorMenuButton.menu \
	-tearoff no

   # Clear menu
   .tlMap.fmMain.fmMapColor.mapColorMenuButton.menu delete 0 end
 
   # Fill the map color menu
   for {set i 1} {$i < [expr {$num_colors}]} {incr i} \
   {
	.tlMap.fmMain.fmMapColor.mapColorMenuButton.menu add command \
	   -label "[lindex $color_table($i) 0] [lindex $color_table($i) 1]" \
   	   -background [format "#%02x%02x%02x" \
			[lindex $color_table($i) 4] \
			[lindex $color_table($i) 3] \
			[lindex $color_table($i) 2]] \
	   -command "
				set MapColor [lindex $color_table($i) 0]
				.tlMap.fmMain.fmMapColor.mapColorMenuButton configure \
					-text [lindex $color_table($i) 1] \
					-background  [format "#%02x%02x%02x" \
								 [lindex $color_table($i) 4] \
								 [lindex $color_table($i) 3] \
								 [lindex $color_table($i) 2]]\
					-activebackground [format "#%02x%02x%02x" \
								 	  [lindex $color_table($i) 4] \
								 	  [lindex $color_table($i) 3] \
								 	  [lindex $color_table($i) 2]]
			"		
   }

   # Projection menu button -----------------------------------

   frame .tlMap.fmMain.projFrame
   pack .tlMap.fmMain.projFrame\
	-padx 10 \
	-pady 5 \
	-fill x \
	-expand yes

   label .tlMap.fmMain.projFrame.label \
	-text "Map Projection: "
   pack .tlMap.fmMain.projFrame.label \
	-side left 

   menubutton .tlMap.fmMain.projFrame.menuButton \
	-text $MapProjection \
	-width 6 \
	-relief raised \
	-borderwidth 2 \
	-indicatoron yes \
	-menu .tlMap.fmMain.projFrame.menuButton.menu
   pack .tlMap.fmMain.projFrame.menuButton \
	-side left

   menu .tlMap.fmMain.projFrame.menuButton.menu \
	-tearoff no
   .tlMap.fmMain.projFrame.menuButton.menu add command \
	-label "MERC" \
	-command ".tlMap.fmMain.projFrame.menuButton configure \
			-text MERC
		  set MapProjection MERC"
   .tlMap.fmMain.projFrame.menuButton.menu add command \
	-label "SAT" \
	-command ".tlMap.fmMain.projFrame.menuButton configure \
			-text SAT
		  set MapProjection SAT"
   .tlMap.fmMain.projFrame.menuButton.menu add command \
	-label "CONF" \
	-command ".tlMap.fmMain.projFrame.menuButton configure \
			-text CONF
		  set MapProjection CONF"

   # Lat/Lon check button -----------------------------------

   checkbutton .tlMap.fmMain.btLALO \
	-text "Draw LAT/LON Lines" \
	-variable MapLALO \
	-onvalue 1 \
	-offvalue 0 \
	-highlightthickness 0 \
	-command { LatLonOptions }
    pack .tlMap.fmMain.btLALO \
	-side top \
	-pady 5 \
	-padx 10 \
	-anchor w

  # Reset to Defaults button
  button .tlMap.fmMain.restoreDefaultsButton \
	-text "Restore Default Settings" \
	-command RestoreDefaultSettings
  pack .tlMap.fmMain.restoreDefaultsButton \
	-side top \
	-padx 5 \
	-pady 5

  # Draw/Dismiss... buttons
  if { [string match "IMAGESELECT" $option] } \
  {
     # The MAP GUI has been called by the ImageSelect GUI
     # --------------------------------------------------

	# Set the map name to the default each time it comes up
	.tlMap.fmMain.fmNameMenu.mb.menu invoke 0

	# Set the projection to SAT and get rid of controls
	.tlMap.fmMain.projFrame.menuButton.menu invoke 1
	pack forget .tlMap.fmMain.projFrame

	# Frame for buttons at the bottom
    	frame .tlMap.fmControls
    	pack .tlMap.fmControls \
	   -side top 

	# Set frame to 'Current' and get rid of the frame
	# selection controls
        .tlMap.fmMain.fmWhichFrames.current invoke
	pack forget .tlMap.fmMain.fmWhichFrames

	# Get rid of the 'Restore Default Settings' button
	pack forget .tlMap.fmMain.restoreDefaultsButton


      	button .tlMap.fmControls.btDismiss \
	   -text "Select" \
	   -command "destroy .tlMap"
      	pack .tlMap.fmControls.btDismiss \
	   -side top \
	   -pady 5
  } \
  elseif { [string match "NODRAW" $option] } \
  {
    	frame .tlMap.fmControls
    	pack .tlMap.fmControls \
	   -side top 

      	button .tlMap.fmControls.btDismiss \
	   -text "Select" \
	   -command "destroy .tlMap"
      	pack .tlMap.fmControls.btDismiss \
	   -side top \
	   -pady 5    
  } \
  else \
  {  
    	frame .tlMap.fmControls 
	pack .tlMap.fmControls \
	   -side top \
	   -padx 5

	button .tlMap.fmControls.btDraw \
	   -text "Draw Map" \
	   -command "DrawMap"
	pack .tlMap.fmControls.btDraw \
	   -side left \
	   -pady 5

      	button .tlMap.fmControls.btDrawDismiss \
	   -text "Draw & Dismiss" \
	   -command "DrawMap; destroy .tlMap"
	pack .tlMap.fmControls.btDrawDismiss \
	   -side left \
	   -padx 5 \
	   -pady 5

       	button .tlMap.fmControls.btDismiss \
	   -text "Dismiss" \
	   -command "destroy .tlMap"
      	pack .tlMap.fmControls.btDismiss \
	   -side right \
	   -pady 5    
  }

  # Hide the lat/lon controls if first startup of map GUI
  if { ![info exists MapInit] } \
  {
#	.tlMap.fmMain.btLALO deselect
#	set MapLALO 0
  }

  LatLonOptions

}  
  

#-------------------------------------------------------------------------------#
# ToggleLALO - displays/hides lat/lon widgets depending upon the status
#	       of the "Draw LAT/LON Lines" check button.
#-------------------------------------------------------------------------------#
proc ToggleLALO {} \
{
  global MapLALO

  if { $MapLALO } \
  {
	pack .tlMap.fmMain.fmLALOOn \
	   -after .tlMap.fmMain.btLALO \
	   -pady 5
  } \
  else \
  { 
	pack forget .tlMap.fmMain.fmLALOOn
  }
}  


#-------------------------------------------------------------------------------#  
# MapColorsUpdateFrame { } -- when the frame is altered and this window 		#
#					  is active then update the color level buttons		#	
#-------------------------------------------------------------------------------#
proc MapColorsUpdateFrame { } \
{
  global MapInit
  global LMActiveFrame
  global MapColor MapLALOColor
  
  if { ![winfo exists .tlMap] } { return }

  .tlMap.fmMain.fmLALOOn.fmLALOColor.btLALOColor config \
	-foreground [ColorLevelToRGB $MapLALOColor ]

  .tlMap.fmMain.fmMapColor.btMapColor config \
	-foreground [ColorLevelToRGB $MapColor ] 
}
  

#------------------------------------------------------------------------------#  
# DrawMap {} -- Draw map(s) based on user input.                               #
#------------------------------------------------------------------------------#
proc DrawMap {} \
{
  global MapName 
  global MapCommandName
  global MapColor 
  global MapLALO 
  global MapLALOColor 
  global MapLALOLonInt 
  global MapLALOLatInt
  global MapInit
  global MapProjection
  global LMActiveLoop 
  global LMLoops
  global LMActiveFrame 
  global LMFrames
  global which_frames

  # Set lat/lon strings
  if { $MapLALO } \
  {
	set LALOString "LALO"
	set LALOColorString $MapLALOColor
	set LALOIntString "INT=$MapLALOLatInt $MapLALOLonInt"
  } \
  else \
  {
	set LALOString ""
	set LALOIntString ""
	set LALOColorString ""
  }

  # Set map projection string
  set MapProjectionString "PRO=$MapProjection"

  # Set range of frames onto which a map is to be drawn.  Since
  # it is possible to set up a loop of discontinuous frames it
  # is necessary to create a list containing all frames onto
  # which the map is to be drawn.
  set frame_list [list]
  if {[string match $which_frames "CURRENT"]} \
  {
	set current_frame [UCpeek 51]
	set beg_range $current_frame
	set end_range $current_frame
	lappend frame_list $current_frame
  } \
  elseif {[string match $which_frames "RANGE"]} \
  {
	set beg_range [.tlMap.fmMain.fmWhichFrames.rangeFrame.enFrom get]
	set end_range [.tlMap.fmMain.fmWhichFrames.rangeFrame.enTo get]

	# Display error message if no values have been entered
	set beg_range [string trim $beg_range]
	set end_range [string trim $end_range]
	if {![string length $beg_range] || ![string length $end_range]} \
	{
	   tk_messageBox \
		-message "If you select 'Range of Frames...' you must \
					enter beginning and ending frame numbers \
					in the entry windows." \
		-icon error \
		-parent .tlMap

	   return
	}

	for {set i $beg_range} {$i <= $end_range} {incr i} \
	{
	   lappend frame_list $i
	}
  } \
  elseif {[string match $which_frames "LOOP"]} \
  {
	# Display error message if the current loop is not 
	# within a defined loop
	if { $LMActiveLoop < 0 } \
	{
	   tk_messageBox \
		-message "The current frame must be within a defined loop\
					in order to use the 'Set For Current Loop' option." \
		-icon error \
		-title "Error" \
		-parent .tlMap

	   return
	}

	foreach i $LMLoops($LMActiveLoop,O) \
	{
	   lappend frame_list [expr {$i + 1}]
	}
  }  

  # Loop through specified frames, drawing a map on each one
  foreach i $frame_list \
  {
	# Call the MAP command
	RunCommand "MAP \
				$MapCommandName \
				$MapColor \
				$LALOString \
				$LALOColorString \
				$LALOIntString \
				$MapProjectionString \
				IMA=$i \
				GRA=$i"

	puts "MAP \
				$MapCommandName \
				$MapColor \
				$LALOString \
				$LALOColorString \
				$LALOIntString \
				$MapProjectionString \
				IMA=$i \
				GRA=$i"
  }
}


#------------------------------------------------------------------------------------#
# GetMaps - Reads Map.USER, Map.SITE, and Map.CORE in that order and returns a list  #
#	    of unique map definitions.                                               #
#------------------------------------------------------------------------------------#
proc GetMaps {} \
{
   global env

   set map_definition_list [list]
   set map_file_list "MAP.USER MAP.SITE MAP.CORE"

   # Insert "VH", and "H" into the map list
   lappend map_definition_list "VH (Continental U.S.)"
   lappend map_definition_list "H (N. America)"

   # Read Map.USER, Map.SITE, and, Map.CORE in that order to produce
   # a complete and non-redundant list of map definitions
   foreach map_file $map_file_list \
   {
	# Get the path to the context map file
        set map_file_full_path [GetLocalDataPath $map_file]

	# Open and read the file if it exists
	if {[file exists $map_file_full_path]} \
   	{
	   set inputOperator [open "|cat $map_file_full_path" r]
	   while {[gets $inputOperator line] >= 0} \
	   {
		# Weed out all unwanted lines
	   	if {![string match [string range [lindex $line 0] 0 0] "#"] && \
	 	    [string length $line] && \
	     	    ![string match [string range $line 0 0] " "] && \
			![string match [string range $line 0 0] "\t"] && \
		    ![string match [lindex $line 0] "GLOBAL"] && \
		    ![string match [lindex $line 0] "EXAMPLE1"] && \
		    ![string match "*.X" [lindex $line 0]]} \
	   	{
			# Add map definition to the list only if it does
			# not already exist in the list
			if {[lsearch $map_definition_list [lindex $line 0]] == -1} \
			{
			   lappend map_definition_list [lindex $line 0]
			}
	   	}
	   }

	   catch {close $inputOperator}
    	}
    }

    # Separate the U.S. states from the rest of the list, sort the lists
    # and splice them back together
    set AK_index [lsearch $map_definition_list "AK"]
    set WY_index [lsearch $map_definition_list "WY"]
    set main_list [lrange $map_definition_list 0 [expr {$AK_index - 1}]]
    set main_list "$main_list [lrange $map_definition_list [expr {$WY_index + 1}] end]"
    set states_list [lrange $map_definition_list $AK_index $WY_index]
    set main_list [lsort $main_list]
    set states_list [lsort $states_list]
    set final_list "$main_list $states_list"

    # Add "Default Map" to the beginning of the list
    set final_list "{Default Map} $final_list"

    return $final_list
}


#-----------------------------------------------------------------------------------#
# FillMapMenu - Populate the 'Map Name' menu.                                       #
#-----------------------------------------------------------------------------------#
proc FillMapMenu { MapMenu } \
{
   global MapName
   global MapCommandName


   # Create menu objects
   menu $MapMenu \
	-tearoff no
   menu $MapMenu.stateMenu \
	-tearoff no

   # Get a list of map definitions from the MAP.* context files
   set map_definitions [GetMaps]

   # Add map definitions from the MAP.* context files to the menu
   set counter 1
   set new_column no
   set state_flag FALSE
   foreach map $map_definitions \
   {
	# Switch to sub-menu if we have reached the U.S. states
	if {[string match $map "AK"]} \
	{
   	   $MapMenu add cascade \
		-label "U.S. State Maps" \
		-menu $MapMenu.stateMenu
	   set state_flag TRUE
	}

	# Add items to the main 'Map Name' menu unless we
	# have reached the U.S. states in which case switch
	# to the sub-menu
	if {[string match $state_flag "FALSE"]} \
	{
	   if { [string match [lindex $map 0] "Default"] } \
	   {
		set map_command "X"
	   } \
	   elseif { [string match [lindex $map 0] "H"] } \
	   {
		set map_command "H"
	   } \
	   elseif { [string match [lindex $map 0] "VH"] } \
	   {
		set map_command "VH"
	   } \
	   else \
	   {
		set map_command $map
	   }

	   $MapMenu add command \
	   	-label $map \
		-command "set MapName \"$map\"
				  set MapCommandName $map_command"
	} \
	else \
	{
	   $MapMenu.stateMenu add command \
	   	-label $map \
		-command "set MapName $map
				  set MapCommandName $map" \
		-columnbreak $new_column

	   if {$counter == 5} \
	   {
	   	set new_column yes
	   	set counter 1
	   } \
	   	else \
	   {
	  	set new_column no
	   	incr counter
	   }
	}
    }
}

#-----------------------------------------------------------------------------------#
# FrameRange - Enables/disables entry windows in frame selection section
#-----------------------------------------------------------------------------------#
proc FrameRange { action } \
{
   global tk_version

   # Tk widget commands moved to namespaces with version 8.1 and higher
   # Expose the old private Tk commands.
   if { $tk_version >= 8.1 } {
     ::tk::unsupported::ExposePrivateCommand tk*
   }

   if {[string match $action "DISABLE"]} \
   {
	.tlMap.fmMain.fmWhichFrames.rangeFrame.enFrom delete 0 end 
	.tlMap.fmMain.fmWhichFrames.rangeFrame.enFrom configure \
		-background gray \
		-state disabled
	.tlMap.fmMain.fmWhichFrames.rangeFrame.enTo delete 0 end 
	.tlMap.fmMain.fmWhichFrames.rangeFrame.enTo configure \
		-background gray \
		-state disabled
   } \
   elseif {[string match $action "ENABLE"]} \
   {
	.tlMap.fmMain.fmWhichFrames.rangeFrame.enFrom configure \
	  	-state normal \
		-background white
	.tlMap.fmMain.fmWhichFrames.rangeFrame.enTo configure \
	  	-state normal \
		-background white
	tkTabToWindow .tlMap.fmMain.fmWhichFrames.rangeFrame.enFrom
   }
}


#-----------------------------------------------------------------------------------#
# SearchContextFile - Returns the value of the given global variable in the given 
#			  context file.  Returns 'NONE' if the given context file
#			  doesn't exist or the given variable was not found.
#-----------------------------------------------------------------------------------#
proc SearchContextFile {context_variable context_file} \
{
    set context_file_full_path [GetLocalDataPath $context_file]
   
    # Return 'NONE' if the context file doesn't exist
    if {![file exists $context_file_full_path]} {return "NONE"}

    # Return 'NONE' if the context file exists but is unreadable
    if {![file readable $context_file_full_path]} {return "NONE"}

    # The context file exists so open it
    set inputOperator [open "$context_file_full_path" r]

    # Initialize a couple of flags
    set line_with_variable "NotFound"
    set index_of_context_variable -1

    # Begin reading one line at a time
    while { [gets $inputOperator line] >= 0 && ![eof $inputOperator] } \
    {
	# If first word in 'line' is 'GLOBAL' then check for the given
	# context variable
	if {[string match [lindex $line 0] "GLOBAL"]} \
	{
	   # Look for the given context variable in the first line
	   set index_of_context_variable [string first $context_variable $line]
	   if {$index_of_context_variable >= 0} \
	   {
		set line_with_variable $line
	   }\
	   else \
	   {
	   	# Search the remaining lines for the context variable.  The search 
	   	# will continue for one line past the last line that ends with 
	   	# a slash because the 'while' loop below checks for the slash first
	   	# and THEN reads another line (simple huh?).
	   	while { [string last "\\" $line] && \
                        [gets $inputOperator line] && \
                        ![eof $inputOperator]} \
	   	{
		   	set index_of_context_variable \
				[string first $context_variable $line]
		   	if {$index_of_context_variable >= 0} \
			{
		   		set line_with_variable $line
		   		break
			}
	   	}
	   }
	}
    }
    catch {close $inputOperator}

    # Return 'NONE' if the given global context variable was not found
    if {$index_of_context_variable == -1} {return "NONE"}

    # Get section of the line after the equal sign
    set index_of_equal_sign [string first "=" $line_with_variable]
    set value_of_context_variable [string range $line_with_variable \
							[expr {$index_of_equal_sign + 1}] end]

    # Remove the slash from the end of the line (if there is one)
    set value_of_context_variable [string trim $value_of_context_variable "\\"]

    # Remove any spaces from the beginning and the end (what remains is the
    # value for the context variable)
    set value_of_context_variable [string trim $value_of_context_variable]

    # Return 'NONE' if the given global context variable was found with value 'X'
    if { [string match $value_of_context_variable "X"] } { return "NONE" }

    return $value_of_context_variable
}


#--------------------------------------------------------------------------------#
# ReadContextFileVariable - Returns the value of the given variable from the
#					map context files.  Checks all three files (MAP.USER, 
#					MAP.SITE, and MAP.CORE) in that order and takes the 
#					value of the first occurance of the variable.
#					Returns 'NONE' if no variable was found.
#--------------------------------------------------------------------------------#
proc ReadContextFileVariable {context_variable} \
{
   foreach context_file {MAP.USER MAP.SITE MAP.CORE} \
   {
	set search_result [SearchContextFile $context_variable $context_file]
	if {![string match $search_result "NONE"]} {return $search_result}
   }

   # Return 'NONE' if the context files were not found or the variable
   # was not found within the context files
   return "NONE"
}


#------------------------------------------------------------------------------#
# IsImageInFrame - Checks to see if an image is currently loaded into a given
#		   frame, returns YES or NO.
#------------------------------------------------------------------------------#
proc IsImageInFrame {frame} \
{
   set inputOperator [open "|frmlist.k $frame" r]
   while {[gets $inputOperator line] >= 0} \
   {
	if {[string match $line "*No frame information*"]} \
	{
	   return "NO"
	}
   }
   catch {close $inputOperator}

   return "YES"

}

#------------------------------------------------------------------------------#
# ReadColorTable
#------------------------------------------------------------------------------#
proc ReadColorTable {} \
{
   global num_colors
   global color_table


   set inputOperator [open "|gu.k TABLE" r]
   gets $inputOperator line; # header line
   gets $inputOperator line; # header line
   set num_colors 0
   while {[gets $inputOperator line] >= 0} \
   {
	if {[string compare [lindex $line 1] "Done"] == 0} {break}

	# Insert 'User-Defined' if no name for color is given
	if {[expr {[llength $line] < 5}] == 1} \
	{
	   set line [linsert $line 1 "User-Defined"]
	}

	set color_table([lindex $line 0]) $line
	incr num_colors
   }
   catch {close $inputOperator}
}


#------------------------------------------------------------------------------#
# Restore Default Settings - resets all map settings to their defaults
#------------------------------------------------------------------------------#
proc RestoreDefaultSettings {} \
{
   global MapProjection
   global MapLALOLonInt 
   global MapLALOLatInt


   # Reset map name
   .tlMap.fmMain.fmNameMenu.mb.menu invoke 0

   # Reset frame selection radio buttons
   .tlMap.fmMain.fmWhichFrames.current invoke

   # Reset map color
   set default_map_color [ReadContextFileVariable MCOLOR]
   if {[string match $default_map_color "NONE"]} \
   {
	.tlMap.fmMain.fmMapColor.mapColorMenuButton.menu invoke 0
   } \
   else \
   {
	.tlMap.fmMain.fmMapColor.mapColorMenuButton.menu invoke \
	[expr {$default_map_color - 1}]
   }

   # Reset map projection
   set MapProjection [ReadContextFileVariable PROJECTION]
   if {[string match $MapProjection "NONE"]} \
   {
	.tlMap.fmMain.projFrame.menuButton.menu invoke 0
   } \
   else \
   {
	.tlMap.fmMain.projFrame.menuButton configure \
		-text $MapProjection
   }

   # Reset Lat/Lon line options
   .tlMap.fmMain.btLALO deselect
   ToggleLALO
   catch {.tlMap.fmMain.fmLALOOn.fmLALOColor.laloColorMenuButton.menu invoke 1}
   set MapLALOLatInt 30
   set MapLALOLonInt 30
}

#------------------------------------------------------------------------------#
# LatLonOptions - Creates lat/lon options section when check button is 
#		  selected for the first time.  Thereafter the options
#		  section is toggled on/off.
#------------------------------------------------------------------------------#
proc LatLonOptions {} \
{
  global MapLALOColor
  global MapLALO
  global MapLALOLatInt
  global MapLALOLonInt
  global num_colors
  global color_table


if { $MapLALO } \
{
   if {[winfo exists .tlMap.fmMain.fmLALOOn]} \
   {

	pack .tlMap.fmMain.fmLALOOn \
	   -side top \
	   -after .tlMap.fmMain.btLALO \
	   -anchor w \
	   -padx 10 \
	   -pady 5 \
	   -expand yes \
	   -fill x
   } \
   else \
   {   
    # This is the first time that the Lat/Lon options check button
    # has been selected so create the Lat/Lon options section.

    frame .tlMap.fmMain.fmLALOOn \
	-relief groove \
	-borderwidth 3
    pack .tlMap.fmMain.fmLALOOn \
	-side top \
	-after .tlMap.fmMain.btLALO \
	-padx 10 \
	-pady 5 \
	-expand yes \
	-fill x

    set f .tlMap.fmMain.fmLALOOn
 
    frame $f.fmLALOColor
    pack $f.fmLALOColor \
	-side top \
	-pady 5 \
	-anchor w

    label $f.fmLALOColor.lbLALOColor \
	-text "Lat/Lon Line Color:" 
    pack $f.fmLALOColor.lbLALOColor \
	-side left \
	-padx 5

   # LALO color menu button
   menubutton $f.fmLALOColor.laloColorMenuButton \
	-text [ColorLevelToName $MapLALOColor] \
	-background [ColorLevelToRGB $MapLALOColor] \
	-activebackground [ColorLevelToRGB $MapLALOColor] \
	-width 12 \
	-relief raised \
	-borderwidth 2 \
	-indicatoron yes \
	-menu $f.fmLALOColor.laloColorMenuButton.menu
   pack $f.fmLALOColor.laloColorMenuButton \
	-side left \
	-padx 5

   # Create and fill lalo color menu
   menu $f.fmLALOColor.laloColorMenuButton.menu \
	-tearoff no

   # Clear menu
   $f.fmLALOColor.laloColorMenuButton.menu delete 0 end
 
   # Fill the map color menu
   for {set i 1} {$i < [expr {$num_colors}]} {incr i} \
   {
	$f.fmLALOColor.laloColorMenuButton.menu add command \
	   -label "[lindex $color_table($i) 0] [lindex $color_table($i) 1]" \
   	   -background [format "#%02x%02x%02x" \
			[lindex $color_table($i) 4] \
			[lindex $color_table($i) 3] \
			[lindex $color_table($i) 2]] \
	   -command "
				set MapLALOColor [lindex $color_table($i) 0]
				$f.fmLALOColor.laloColorMenuButton configure \
					-text [lindex $color_table($i) 1] \
					-background [format "#%02x%02x%02x" \
								[lindex $color_table($i) 4] \
								[lindex $color_table($i) 3] \
								[lindex $color_table($i) 2]] \
					-activebackground [format "#%02x%02x%02x" \
									  [lindex $color_table($i) 4] \
									  [lindex $color_table($i) 3] \
									  [lindex $color_table($i) 2]]
			"		
   }

   frame $f.fmLALOLatInt
   pack $f.fmLALOLatInt \
	-side top \
	-pady 10 \
	-anchor w \
	-fill x \
	-expand true

    label $f.fmLALOLatInt.lbLALOLatInt \
	-text "Line Interval -- Latitude:"
    pack $f.fmLALOLatInt.lbLALOLatInt \
	-side left \
	-padx 5

    entry $f.fmLALOLatInt.enLALOLatInt \
	-textvariable MapLALOLatInt \
	-width 5 \
	-background white        
    pack $f.fmLALOLatInt.enLALOLatInt \
	-side right \
	-padx 5 

    frame $f.fmLALOLonInt
    pack $f.fmLALOLonInt \
	-side top \
	-pady 3 \
	-anchor w \
	-fill x \
	-expand true

    label $f.fmLALOLonInt.lbLALOLonInt \
	-text "Line Interval -- Longitude:"
    pack $f.fmLALOLonInt.lbLALOLonInt \
	-side left \
	-padx 5

    entry $f.fmLALOLonInt.enLALOLonInt \
	-textvariable MapLALOLonInt \
	-width 5 \
	-background white
    pack $f.fmLALOLonInt.enLALOLonInt \
	-side right \
	-padx 5
   }
} \
else \
{ 
   pack forget .tlMap.fmMain.fmLALOOn
}

}

