# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: mctokan.tcl,v 1.7 1998/10/13 20:26:44 dglo Exp $

#$ Name:
#$      mcTokanizeString - tokanizes a McIDAS command string into
#$                         its components for Tcl
#$
#$ Interface:
#$      mcTokanizeString { char *string }
#$
#$ Input:
#$      string     - character string to be parsed and tokanized
#$
#$ Input and Output:
#$      none
#$
#$ Output:
#$      none
#$
#$ Return values:
#$
#$
#$ Remarks:
#$      The resulting string is tokanized into a series of global variables
#$ as follows:
#$
#$      mc_keywords    - list of all keywords that were found. If command
#$                       parameters are found, a special keyword " "
#$                       will occur on the front of the list.
#$      mc_numparms    - number of positional parameters for each keyword
#$      mc_posparms    - list of all the positional parameters
#$      mc_double      - double quote string value
#$
#$      You can then use the proc mccmdstr to extract individual parameters
#$ from the tokanized string.

proc mcTokanizeString { s } {

  global          mc_keywords
  global          mc_numparms
  global          mc_posparms
  global          mc_double

# if the global variables previously existed, cream them

  if [info exists mc_keywords] {
    unset mc_keywords
  }
  if [info exists mc_numparms] {
    unset mc_numparms
  }
  if [info exists mc_posparms] {
    unset mc_posparms
  }
  if [info exists mc_double] {
    unset mc_double
  }

  set  strlen      [string length $s]

  set  i           0

  set  quotestat   0
  set  doubleq     0
  set  status      0

  set  nposparm    0
  set  nkeywords   0
  set  mc_double   " "
  list mc_numparms 0

  # status  = 0      not currently building a token
  # status  = 1      building a token
  # status  = 2      token just created is a parameter
  # status  = 3      token just created is a keyword
  # status  = 4      token is a double quote string

  # loop through each of the characters in the string

  while {$i < $strlen} {

    # extract the ith character of the string

    set char [string index $s $i]

    # if the character is a-z0-9 or single of double quote mark
    # check to see if this is could change the status flag

    if {[regexp {[a-zA-Z0-9\'\"]+} $char]} {

      # if we haven't started building a tokan yet

      if {$status == 0} {

	# set the status to 'building a tokan' and assign the starting
	# character location for the tokan

	set status 1
	set begtok $i

	# if the character is actually a single or double quote mark
	# start the tokan on the next character, not the current one

	if { ($char == "'") || ($char == "\"")} {
	  set begtok [expr $i + 1]
	}
      }

      # if it is a single quote, update the quote status

      if {$char == "'"} {
	set quotestat [expr $quotestat + 1]
      }

      # if it is a double quote, update the status

      if {$char == "\""} {
	set doubleq 1
      }
    }

    # if we are currently building a tokan and we get a special
    # character we may have to update the status flag

    if {$status == 1} {

      switch -- $char {

       "=" {
	     # if keyword flag is found, update the status if you are
	     # not in a single or double quote string

	     if {($quotestat == 0) && ($doubleq != 1)} {
	       set status 3
	     }
	   }
       "," {
	     if {($quotestat == 0) && ($doubleq != 1)} {
	       set status 3
	     }
	   }

       " " {
	     # if a blank is found and we are not in a quote string
	     # bind off the tokan as a positional parameter

	     if {($quotestat == 0) && ($doubleq != 1)} {
	       set status 2
	     }
	   }

       "'" {
	     # if this is a single quote and we already have a single
	     # quote, bind off the parameter

	     if {$quotestat == 2} {
	       set status 2
	     }
	   }
      }

      # if we are at the last character in the string, bind off the
      # parameter

      if {$i == [expr $strlen - 1]} {
	set status 2

	# change the status to double quote string if the flag has
	# been set

	if {$doubleq == 1} {
	  set status 4
	}
	incr i
      }
    }

    # if some type of parameter has been bound off

    if {$status > 1} {

      # set the ending character location

      set endtok [expr $i - 1]

      # extract the parameter

      set param [string range $s $begtok $endtok]

#     if a positional parameter is found

      if {$status == 2} {

       if {$nposparm > 0} {
	 set nk [expr $nkeywords - 1]
	 set mc_numparms [lreplace $mc_numparms $nk $nk \
			  [expr $nposparm + 1]]
	} else {
	  lappend mc_numparms 1
	  if {$nkeywords == 0} {
	    lappend mc_keywords " "
	    set nkeywords [expr $nkeywords + 1]
	  }
	}
	lappend mc_posparms $param
	set nposparm [expr $nposparm + 1]

      }

#     if a keyword is found

      if {$status == 3} {

	set nkeywords [expr $nkeywords + 1]
	set nposparm 0
	lappend mc_keywords $param

      }

#     if a double quote string is found

      if {$status == 4} {
	set mc_double $param
      }

      # reinitialize the status flags and continue

      set status 0
      set quotestat 0
    }
    incr i
  }
}

#$ Name:
#$      mccmdstr - Tcl proc to extract parameters from a tokanized
#$                 command parameter string
#$
#$ Interface:
#$      mccmdstr (char keyword, int position, char default, char value)
#$
#$ Input:
#$      keyword    - keyword to get value for. use " " for command
#$                   positional parameters
#$      position   - position number to extract (1-based)
#$      default    - default value to return
#$
#$ Input and Output:
#$      none
#$
#$ Output:
#$      value      - string value returned
#$
#$ Return values
#$      0          - success
#$     -1          - keyword not found
#$     -2          - position number not found
#$
#$ Remarks:
#$      Before values can be extracted with this proc you must call
#$ the proc mcTokanizeString.
#$      You can use the McIDAS syntax for optional keyword characters.
#$ For example "DAT.ASET"

proc mccmdstr {keyword position default value} {

  global          mc_keywords
  global          mc_numparms
  global          mc_posparms
  global          mc_double
  upvar           $value      val

  set nkeywords   [llength $mc_keywords]
  set kwpos      -1
  set posoffset   0
  set val         $default

# search for the keyword name

  for {set i 0} {$i < $nkeywords} {incr i} {

#   build the keyword name to test from the list of keywords

    set test_kw [lindex $mc_keywords $i]

#   if a keyword match has been found, set a flag and exit the for loop

    if { [KeywordsMatch $test_kw $keyword] == 1 } {
      set kwpos $i
      set maxposparm [lindex $mc_numparms $i]
      break
    }

#   set the position offset on mc_posparms where, potentially,
#   the list of parameters for the correct keyword is found

    set posoffset [expr $posoffset + [lindex $mc_numparms $i]]

  }

# if no keyword match has been found, error out

  if {$kwpos == -1} {
    return -1
  }

# now we have the correct keyword, try to get the appropriate
# positional parameter

  if { $position > $maxposparm } {
    return -2
  }

# extract the string value

  set offset [expr $posoffset+ $position - 1]
  set val [lindex $mc_posparms $offset]
  return 0

}

#$ Name:
#$      mccmdnum - Tcl proc to extract number of parameters for a
#$                 keyword
#$
#$ Interface:
#$      mccmdnum (char keyword)
#4
#$ Input:
#$      keyword    - keyword to get value for. use " " for command
#$                   positional parameters
#$ Input and Output:
#$      none
#$
#$ Output:
#$      none
#$
#$ Return values
#$      n          - number of parameters
#$     -1          - keyword not found
#$
#$ Remarks:
#$      Before values can be extracted with this proc you must call
#$ the proc mcTokanizeString.
#$      You can use the McIDAS syntax for optional keyword characters.
#$ For example "DAT.ASET"

proc mccmdnum {keyword} {

  global          mc_keywords
  global          mc_numparms

  set nkeywords   [llength $mc_keywords]
  set kwpos      -1

# search for the keyword name

  for {set i 0} {$i < $nkeywords} {incr i} {

#   build the keyword name to test from the list of keywords

    set test_kw [lindex $mc_keywords $i]

#   if a keyword match has been found, set a flag and exit the for loop

    if { [KeywordsMatch $test_kw $keyword] == 1 } {
      set maxposparm [lindex $mc_numparms $i]
      return $maxposparm
    }

  }

# if no keyword match has been found, error out

  return -1

}

#$ Name:
#$      mccmdkeystring - Tcl proc to extract the entire argument string
#$                       for a particular keyword
#$
#$ Interface:
#$      mccmdkeystring (char keyword, char string)
#4
#$ Input:
#$      keyword    - keyword to get value for. use " " for command
#$                   positional parameters
#$ Input and Output:
#$      none
#$
#$ Output:
#$      string     - entire argument string for a specified keyword
#$
#$ Return values
#$      0          - success
#$     -1          - no keyword found
#$
#$ Remarks:
#$      Before values can be extracted with this proc you must call
#$ the proc mcTokanizeString.
#$      You can use the McIDAS syntax for optional keyword characters.
#$ For example "DAT.ASET"

proc mccmdkeystring {keyword outstring} {

  upvar           $outstring  stringval
  global          mc_keywords
  global          mc_numparms

  set nparms   [mccmdnum $keyword]

  if {$nparms <= 0} {
    return -1
  }

  set buildstring ""

# search for the keyword name

  for {set i 1} {$i <= $nparms} {incr i} {

    mccmdstr $keyword $i " " tempstr

    set buildstring "$buildstring $tempstr"

  }

  set stringval [string trimleft $buildstring]

  return 0

}

# list the contents of a string that has been run through the proc
# mcTokanize. this is really not used by applications. it is really
# for debugging.

proc mcListTokans  {} {

  global          mc_keywords
  global          mc_numparms
  global          mc_posparms
  global          mc_double

  set len [llength $mc_keywords]

  set t 0
  for {set i 0} {$i < $len} {incr i} {
    puts stdout "keyword='[lindex $mc_keywords $i]'"

    set np [lindex $mc_numparms $i]
    for {set j 0} {$j < $np} {incr j} {
      set key [lindex $mc_posparms $t]
      puts stdout "         $key"
      incr t
    }
  }

  puts stdout "Double quote =|$mc_double|"
}

#* Name:
#*      KeywordsMatch - determines whether a string matches for
#*                      the mcidas syntax
#*
#* Interface:
#*      KeywordsMatch testkw mckeyword
#*
#* Input:
#*      testkw        - keyword string to test
#*      mckeyword     - keyword string in mcidas syntax
#*
#* Input and Output:
#*      none
#*
#* Output:
#*      none
#*
#* Return values
#*      1             - keywords match
#*      0             - keywords don't match
#*
#* Remarks:
#*      Example:  testkw = 'DATAS' and mckeyword='DAT.ASET' would
#* result in a return value of 1

proc KeywordsMatch {cmdkw keyword} {

# determine if a dot is found in the mcidas keyword. also determine
# input string lengths and set the first character to be tested

  set dotfound   [string first "." $keyword ]
  set cmdkwlen   [string length $cmdkw]
  set keywordlen [string length $keyword]
  set begchr     0

# if a dot is found, check the first n characters up to the dot to
# determine that we have a complete match to that point

  if {$dotfound > 0} {
    set testcmdkw    [string range $cmdkw 0 [expr $dotfound - 1]]
    set testkeyword  [string range $keyword 0 [expr $dotfound - 1]]

#   keyword mismatch

    if {[string compare $testcmdkw $testkeyword] != 0} {
      return 0
    }

#   if we have a complete match to this point, adjust the value of
#   begchr to begin the secondary search after the dot character

    set begchr [expr $dotfound + 1]
  }

# set an ending character for the search

  set endchr     $keywordlen
  if {$endchr > $cmdkwlen} {
    set endchr $cmdkwlen
  }

# scan the remaining characters in the strings looking for exact matches
# character by character

  for {set i $begchr} {$i < $endchr} {incr i} {
    set testch1 [string index $cmdkw $i]
    set testch2 [string index $keyword $i]

#   keyword mismatch

    if {$testch1 != $testch2} {
      return 0
    }
  }

# if we have made it to here, the keywords match

  return 1
}
